package com.starbuds.server.service.message.impl;

import cn.hutool.crypto.digest.HMac;
import cn.hutool.crypto.digest.HmacAlgorithm;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.message.sms.SmsAgentNameConstant;
import com.starbuds.server.common.service.message.SmsPushService;
import com.starbuds.server.service.message.entity.sms.BaiduSms;
import com.starbuds.server.service.message.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;

/**
 * 短信服务-百度
 *
 * @author aoe
 * @date 2021/7/1
 */
@Slf4j
@Service(version = "1.0.0")
public class SmsBaiduServiceImpl implements SmsPushService {

    @Resource
    SmsProvider smsProvider;

    @PostConstruct
    public void init() {
        smsProvider.register(SmsAgentNameConstant.BAIDU, this);
    }

    /**
     * 发送短信（调用方请根据实际情况控制QPS）
     *
     * @param tel          手机号
     * @param varMap       变量Map
     * @param templateId   短信模板
     * @param accessKey
     * @param accessSecret
     * @param sign         短信签名
     * @return true 发送请求成功
     */
    @Override
    public boolean send(String tel, Map<String, String> varMap, String templateId, String accessKey, String accessSecret, String sign) {
        // 文档 https://cloud.baidu.com/doc/SMS/s/lkijy5wvf
        BaiduSms sms = new BaiduSms();
        sms.setMobile(tel);
        sms.setTemplate(templateId);
        sms.setSignatureId(sign);
        sms.setContentVar(varMap);

        String path = "/api/v3/sendSms";
        Map<String, String> headers = headers(path, accessKey, accessSecret);
        if (headers == null) {
            log.error("通过百度发送短信失败，构建Headers失败 path={} accessKey={} accessSecret={}", path, accessKey, accessSecret);
            return false;
        }
        String res = HttpUtil.post(baseUrl() + path, JSON.toJSONString(sms), headers);
        JSONObject resBody = JSON.parseObject(res);
        if (resBody == null) {
            log.error("通过百度发送短信失败，返回不是JSON response = " + res);
            return false;
        }
        String code = resBody.getString("code");
        String successCode = "1000";
        if (successCode.equals(code)) {
            JSONArray data = resBody.getJSONArray("data");
            if (data != null && data.getJSONObject(0) != null
                    && successCode.equals(data.getJSONObject(0).getString("code"))) {
                log.info("通过百度发送短信成功 tel = {} response = {}", tel, res);
                return true;
            }
        }
        log.warn("通过百度发送短信失败 tel = {} varMap = {}, templateId = {} response = {}", tel, varMap, templateId, res);
        return false;
    }

    /**
     * 最大Qps
     *
     * @return
     */
    @Override
    public int maxQps() {
        return 50;
    }

    private Map<String, String> headers(String path, String accessKey, String accessSecret) {
        Map<String, String> headers = new HashMap<>();
        String timestamp = timestamp();
        headers.put("x-bce-date", timestamp);
        // bce-auth-v{version}/{accessKeyId}/{timestamp}/{expirationPeriodInSeconds}/{signedHeaders}/{signature}
        String authorization = authorization(path, timestamp, accessKey, accessSecret);
        if (authorization == null) {
            return null;
        }
        headers.put("Authorization", authorization);
        headers.put("Host", "smsv3.bj.baidubce.com");

        // 生成认证字符串 https://cloud.baidu.com/doc/Reference/s/njwvz1yfu
        return headers;
    }

    private String baseUrl() {
        return "http://smsv3.bj.baidubce.com";
    }

    /**
     * 签名
     * 参考：https://cloud.baidu.com/signature/index.html
     *
     * @param path
     * @param timestamp
     * @return 签名失败时返回 null
     */
    private String authorization(String path, String timestamp, String accessKey, String accessSecret) {
        String accessKeyId = accessKey;
        // bce-authStringPrefix-v{version}/{accessKeyId}/{timestamp}/{expirationPeriodInSeconds}/{signedHeaders}/{signature}
        String authStringPrefix = "bce-auth-v%s/%s/%s/%s";
        authStringPrefix = String.format(authStringPrefix, 1, accessKeyId, timestamp, 1800);
        String signingKey = sha256Hex(accessSecret, authStringPrefix);
        if (signingKey == null) {
            log.error("signingKey = null 百度短信签名失败 accessSecret={} authStringPrefix={}", accessSecret, authStringPrefix);
            return null;
        }
        String canonicalRequest = "POST" + "\n" + path + "\n\n" + "host:smsv3.bj.baidubce.com"
                + "\nx-bce-date:" + StringUtils.replace(timestamp, ":", "%3A");
        String signature = sha256Hex(signingKey, canonicalRequest);
        if (signature == null) {
            log.error("signature = null 百度短信签名失败 signingKey={} canonicalRequest={}", signingKey, canonicalRequest);
            return null;
        }
        String signedHeaders = "host;x-bce-date";
        // authStringPrefix/signedHeaders/signature
        return authStringPrefix + "/" + signedHeaders + "/" + signature;
    }

    private String sha256Hex(String signingKey, String stringToSign) {
        Charset UTF8 = Charset.forName("UTF-8");

        HMac hMac = new HMac(HmacAlgorithm.HmacSHA256, signingKey.getBytes(UTF8));
        return hMac.digestHex(stringToSign);
        /*try {
            Mac mac = Mac.getInstance("HmacSHA256");
            mac.init(new SecretKeySpec(signingKey.getBytes(UTF8), "HmacSHA256"));
            return new String(Hex.encodeHex(mac.doFinal(stringToSign.getBytes(UTF8))));
        } catch (Exception e) {
            log.error("sha256Hex失败", e);
            return null;
        }*/
    }

    private static String timestamp() {
        SimpleDateFormat utcTimeFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
        utcTimeFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
        return utcTimeFormat.format(new Date());
    }

    public static void main(String[] args) {
        String accessKey = "8f032fa017954ef3a08db60473069755";
        String accessSecret = "e352b5b364bf4174b0201d53e0080ba9";
        String sign = "sms-sign-xoingZ89342";
        String templateId = "sms-tmpl-IazGQs91376";
        SmsBaiduServiceImpl s = new SmsBaiduServiceImpl();

        Map<String, String> varMap = new HashMap<>();
        varMap.put("userName", "东华，收到请回答");
        // 悟空 15258836236
        // 博强 17857199457
        // 东华 13456787929
        s.send("13456787929", varMap, templateId, accessKey, accessSecret, sign);

        /*System.out.println(System.getProperty("java.version"));
        Provider[] providers = Security.getProviders();
        String jsonStr = JSON.toJSONString(providers);
        JSONArray array = JSON.parseArray(jsonStr);
//        System.out.println(array);

        for (int i = 0; i < array.size(); i++) {
            JSONObject json = array.getJSONObject(i);
            for (String key : json.keySet()) {
                if (StringUtils.contains(key, "Mac.HmacSHA256")) {
                    System.out.println(key + " " + json.getString(key));
                }
            }
        }*/
    }

}