package com.starbuds.server.service.message.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.starbuds.server.common.constant.message.sms.SmsAgentNameConstant;
import com.starbuds.server.common.service.message.SmsPushService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.util.DigestUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/***
 * @author qinjiang
 * @description 短信服务-悦可
 * @date 2021/9/18
 */
@Slf4j
@Service(version = "1.0.0")
public class SmsYuekeServiceImpl implements SmsPushService {

    //返回结果分析用的分隔符
    private static final String Separator = ",";

    //日志打印前缀
    private static final String LogPrefix = "--SmsYueke短信服务：";

    @Resource
    SmsProvider smsProvider;

    @PostConstruct
    public void init() {
        smsProvider.register(SmsAgentNameConstant.YUEKE, this);
    }

    /**
     * 发送短信（调用方请根据实际情况控制QPS）
     *
     * @param tel          手机号
     * @param varMap       变量Map
     * @param templateId   短信模板
     * @param accessKey
     * @param accessSecret
     * @param sign         短信签名
     * @return true 发送请求成功
     */
    @Override
    public boolean send(String tel, Map<String, String> varMap, String templateId, String accessKey, String accessSecret, String sign) {
        if (StringUtils.isBlank(accessKey) || StringUtils.isBlank(accessSecret)) {
            log.info(LogPrefix + "发送短信失败：accessKey:" + accessKey + " accessSecret:" + accessSecret);
            return false;
        }
        if (StringUtils.isBlank(tel) || tel.length() < 11) {
            log.info(LogPrefix + "发送短信失败，tel:" + tel);
            return false;
        }

        String tempContent = varMap.get("tempContent");
        if (tempContent.contains("${content}")) {
            String content = varMap.get("content");
            if (StringUtils.isBlank(content)) {
                return false;
            }
            tempContent = tempContent.replace("${content}", content);
        }
        if (tempContent.contains("${userName}")) {
            String content = varMap.get("userName");
            if (StringUtils.isBlank(content)) {
                return false;
            }
            tempContent = tempContent.replace("${userName}", content);
        }

        List<String> params = new ArrayList<>();
        params.add(String.format("%s=%s", "user", accessKey));
        params.add(String.format("%s=%s", "passwd", getPasswdDigest(accessKey, accessSecret)));
        params.add(String.format("%s=%s", "msg", tempContent));
        params.add(String.format("%s=%s", "mobs", tel));
        params.add(String.format("%s=%s", "ts", Long.valueOf(System.currentTimeMillis())));
        params.add(String.format("%s=%d", "dtype", 0));
        params.add(String.format("%s=%s", "extno", ""));

        String requestParam = String.join("&", params);
        HttpRequest request = HttpUtil.createGet(String.format("%s?%s", baseURL(), requestParam));
        HttpResponse response = request.execute();
        if (response.isOk()) {
            return analyseMessageResult(response.body());
        }

        log.info(LogPrefix + "发送短信响应失败：" + response.body());
        return false;
    }

    /**
     * 最大Qps
     *
     * @return
     */
    @Override
    public int maxQps() {
        return 120;
    }

    /**
     * 请求发送API
     *
     * @param user
     * @param passwd
     * @return
     */
    private String getPasswdDigest(String user, String passwd) {
        return DigestUtils.md5DigestAsHex((user + passwd).getBytes()).toUpperCase();
    }

    /**
     * 悦可短信发送基本地址
     *
     * @return
     */
    private String baseURL() {
        return "http://211.149.147.79:8080/SendMessage/SendSms_API";
    }

    /**
     * 分析发送消息结果 格式：0,210918173816453986
     *
     * @param responseBody
     * @return
     */
    private boolean analyseMessageResult(String responseBody) {
        if (StringUtils.isBlank(responseBody)) {
            log.info(LogPrefix + "发送短信失败返回为空");
            return false;
        }

        if (!responseBody.contains(Separator)) {
            log.info(LogPrefix + "返回格式错误，rsp:" + responseBody);
            return false;
        }

        String result = responseBody.substring(0, responseBody.indexOf(Separator));
        if (!result.equals("0")) {
            log.info(LogPrefix + "发送短信返回结果失败：" + responseBody);
            return false;
        }

        return true;
    }

}
