package com.starbuds.server.service.message.provider;

import com.alibaba.fastjson.JSON;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.push.model.v20160801.PushRequest;
import com.aliyuncs.push.model.v20160801.PushResponse;
import com.starbuds.server.common.constant.message.push.AndroidNotifyType;
import com.starbuds.server.common.constant.message.push.AndroidOpenType;
import com.starbuds.server.common.constant.message.push.PushTarget;
import com.starbuds.server.common.pojo.message.AliyunConfig;
import com.starbuds.server.common.pojo.message.push.AliyunPushConfig;
import com.starbuds.server.common.pojo.message.push.PushMsg;
import com.starbuds.server.common.service.core.ConfigService;
import com.starbuds.server.common.service.data.AliyunPushConfigService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

/**
 * 阿里云推送服务实现
 *
 * @author Lay
 * @date 2019/06/14
 */
@Component
@Slf4j
public class AliyunPushProvider implements IPushProvider {

    @Reference(version = "1.0.0")
    ConfigService configService;

    @Reference(version = "1.0.0")
    AliyunPushConfigService aliyunPushConfigService;

    /**
     * 推送消息
     *
     * @param pushMsg
     * @param packageName
     * @return
     */
    @Override
    public boolean push(PushMsg pushMsg, String packageName) {

        PushTarget pushTarget = pushMsg.getPushTarget();

        AliyunConfig aliyunConfig = configService.getAliyunConfig();

        if (aliyunConfig == null) {
            log.info("请先配置阿里云accessKey 和 accessSecret");
            return false;
        }

        AliyunPushConfig aliyunPushConfig = aliyunPushConfigService.getAliyunPushConfig(packageName);
        if (aliyunPushConfig == null) {
            log.info("未获取到阿里云推送配置");
            return false;
        }

        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", aliyunConfig.getAccessKey(), aliyunConfig.getAccessSecret());
        IAcsClient client = new DefaultAcsClient(profile);

        PushRequest pushRequest = new PushRequest();
        pushRequest.setTarget(pushTarget.getValue());
        pushRequest.setTargetValue(pushMsg.getTargetValue());
        pushRequest.setPushType(pushMsg.getPushType().getValue());
        pushRequest.setDeviceType(pushMsg.getDeviceType().getValue());
        pushRequest.setTitle(pushMsg.getTitle());
        pushRequest.setBody(pushMsg.getBody());

        String extParam = JSON.toJSONString(pushMsg.getParam());

        // 推送配置 iOS
        // iOS应用图标右上角角标
//        pushRequest.setIOSBadge(1);
        // iOS通知声音
        pushRequest.setIOSMusic("default");
        // 推送交互
//        pushRequest.setIOSNotificationCategory(String.valueOf(pushMsg.getParam().getAction()));
        // 推送副标题
//         pushRequest.setIOSSubtitle(pushMsg.getBody());
        // 是否允许扩展iOS通知内容
        pushRequest.setIOSMutableContent(true);
        // 推送环境
        pushRequest.setIOSApnsEnv(aliyunPushConfig.getPushEnv());
        // 消息推送时设备不在线（既与移动推送的服务端的长连接通道不通），则这条推送会做为通知，通过苹果的APNs通道送达一次。注意：离线消息转通知仅适用于生产环境
        pushRequest.setIOSRemind(true);
        // iOS消息转通知时使用的iOS通知内容，仅当iOSApnsEnv=PRODUCT && iOSRemind为true时有效
        pushRequest.setIOSRemindBody(pushMsg.getBody());
        // 通知的扩展属性
        pushRequest.setIOSExtParameters(extParam);
        // 设置角标
        pushRequest.setIOSBadgeAutoIncrement(true);
        // 静默通知
        pushRequest.setIOSSilentNotification(false);

        // 推送配置 Android
        // 通知声音
        pushRequest.setAndroidMusic("default");
        // 通知的提醒方式
        pushRequest.setAndroidNotifyType(AndroidNotifyType.BOTH.getValue());
        // 通知栏自定义样式0-100
        pushRequest.setAndroidNotificationBarType(50);
        // 通知栏自定义样式0-100
        pushRequest.setAndroidNotificationBarPriority(1);
        // 点击通知后动作
        pushRequest.setAndroidOpenType(AndroidOpenType.Application.getValue());
        // 推送通道设置
        pushRequest.setAndroidNotificationChannel("1");
        // 定通知的扩展属性
        pushRequest.setAndroidExtParameters(extParam);
        // 离线消息是否保存,若保存, 在推送时候，用户即使不在线，下一次上线则会收到
        pushRequest.setStoreOffline(true);

        try {
            pushRequest.setAppKey(aliyunPushConfig.getIosAppKey());
            log.info("-------------------推送[IOS]开始-------------------");
            PushResponse pushResponse = client.getAcsResponse(pushRequest);
            log.info("推送[IOS]：RequestId: {}, MessageID: {}", pushResponse.getRequestId(), pushResponse.getMessageId());
            log.info("-------------------推送[IOS]结束-------------------");

            pushRequest.setAppKey(aliyunPushConfig.getAndroidAppKey());
            pushResponse = client.getAcsResponse(pushRequest);
            log.info("推送[Android]：RequestId: {}, MessageID: {}", pushResponse.getRequestId(), pushResponse.getMessageId());

            return true;
        } catch (Exception e) {
            log.error("推送通知失败", e);
        }

        return false;
    }

}
