package com.starbuds.server.service.message.provider;

import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.message.ImUserCreateDTO;
import com.starbuds.server.common.pojo.message.ImUserProfile;
import com.starbuds.server.common.pojo.message.LiveChatRoomUserProfile;
import com.starbuds.server.common.pojo.message.MsgPacket;
import com.starbuds.server.common.pojo.message.im.BaseImMsg;
import com.starbuds.server.common.pojo.message.im.ImPushInfo;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.pojo.user.UserSimpleInfo;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.message.entity.CustomMsg;
import com.starbuds.server.service.message.entity.RoomJoinMsg;
import com.starbuds.server.service.message.tencent.response.biz.TencentImBizGetAppInfoResponse;
import io.rong.RongCloud;
import io.rong.messages.BaseMessage;
import io.rong.messages.CmdMsgMessage;
import io.rong.messages.TxtMessage;
import io.rong.models.Result;
import io.rong.models.chatroom.ChatroomEntryModel;
import io.rong.models.chatroom.ChatroomMember;
import io.rong.models.chatroom.ChatroomModel;
import io.rong.models.message.ChatroomMessage;
import io.rong.models.message.PrivateMessage;
import io.rong.models.response.*;
import io.rong.models.user.UserModel;
import io.rong.util.CommonUtil;
import io.rong.util.GsonUtil;
import io.rong.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.DelegatingServletInputStream;
import org.springframework.stereotype.Component;

import java.net.HttpURLConnection;
import java.net.URLEncoder;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.starbuds.server.service.message.util.SpringContextUtil.isDev;

/**
 * 融云服务
 *
 * @author Lay
 * @date 2021/3/25
 */
@Component
@Slf4j
public class RongCloudProvider implements IImProvider {

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    private String appKey;
    private String appSecret;
    private RongCloud rongCloud;

    public RongCloudProvider(@Value("${im.rongcloud.appKey}") String appKey, @Value("${im.rongcloud.appSecret}") String appSecret) {
        this.appKey = appKey;
        this.appSecret = appSecret;
        this.rongCloud = RongCloud.getInstance(appKey, appSecret);
        log.info("融云初始化-开始="+rongCloud.getConfig().apiDomains + "c="+rongCloud.getConfig().getDomain());
    }

    /**
     * 创建token
     *
     * @param userId
     * @return
     */
    @Override
    public String buildToken(Long userId) {
        UserCardVO userCard = userService.getNameAndAvatarById(userId);
        ImUserProfile imUserProfile = new ImUserProfile();
        imUserProfile.setUserId(userId);
        imUserProfile.setUserName(userCard.getUserName());
        imUserProfile.setUserAvatar(userCard.getUserAvatar());
        return buildToken(imUserProfile);
    }

    /**
     * 创建token
     *
     * @param imUserProfile
     * @return
     */
    @Override
    public String buildToken(ImUserProfile imUserProfile) {
        if (imUserProfile != null) {
            UserModel userModel = new UserModel()
                    .setId(String.valueOf(imUserProfile.getUserId()))
                    .setName(imUserProfile.getUserName() != null ? imUserProfile.getUserName() : "")
                    .setPortrait(imUserProfile.getUserAvatar() != null ? imUserProfile.getUserAvatar() : "");
            try {
                TokenResult result = rongCloud.user.register(userModel);
                if (!result.getCode().equals(200)) {
                    log.info("融云 Token 创建成功:{}", JSON.toJSONString(result));
                }
                return result.getToken();
            } catch (Exception e) {
                log.error("融云 Token 创建失败", e);
            }
        }
        return null;
    }

    /**
     * 刷新用户资料
     *
     * @param imUserProfile
     * @return
     */
    @Override
    public boolean refreshUser(ImUserProfile imUserProfile) {
        if (imUserProfile == null || imUserProfile.getUserId() == null) {
            return false;
        }

        UserModel userModel = new UserModel()
                .setId(String.valueOf(imUserProfile.getUserId()))
                .setName(imUserProfile.getUserName())
                .setPortrait(imUserProfile.getUserAvatar());
        try {
            Result result = rongCloud.user.update(userModel);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云 Token 获取失败", e);
        }
        return false;
    }

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param content
     * @return
     */
    @Override
    public boolean sendToPeer(Long userId, Long targetId, String content) {
        MsgPacket msgPacket = new MsgPacket();
        msgPacket.setMsg(JSON.parseObject(content));
        msgPacket.setGroupId(String.valueOf(targetId));
        msgPacket.setSenderId(userId);
        msgPacket.setTargetId(targetId);
        msgPacket.setTargetType(ImTargetType.USER);

        CmdMsgMessage cmdMsg = new CmdMsgMessage("ti1mi", toJSON(msgPacket));
        PrivateMessage message = new PrivateMessage()
                .setSenderId(String.valueOf(userId))
                .setTargetId(new String[]{String.valueOf(targetId)})
                .setObjectName(cmdMsg.getType())
                .setContent(cmdMsg);
        try {
            ResponseResult result = rongCloud.message.msgPrivate.send(message);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云发送单聊消息失败:", e);
        }
        return false;
    }

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param data
     * @return
     */
    @Override
    public boolean sendToPeerCustomMsg(Long userId, Long targetId, String data) {
        CustomMsg customMsg = new CustomMsg(data);
        PrivateMessage message = new PrivateMessage()
                .setSenderId(String.valueOf(userId))
                .setTargetId(new String[]{String.valueOf(targetId)})
                .setObjectName(customMsg.getType())
                .setContent(customMsg);
        try {
            ResponseResult result = rongCloud.message.msgPrivate.send(message);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云发送单聊自定义消息失败:", e);
        }
        return false;
    }

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param msg
     * @param pushContent
     * @param isIncludeSender
     * @return
     */
    @Override
    public boolean sendToPeerCustomMsg(Long userId, Long targetId, BaseMessage msg, String pushContent, Integer isIncludeSender) {
        PrivateMessage message = new PrivateMessage()
                .setSenderId(String.valueOf(userId))
                .setTargetId(new String[]{String.valueOf(targetId)})
                .setObjectName(msg.getType())
                .setPushContent(pushContent)
                .setIsIncludeSender(isIncludeSender)
                .setContent(msg);
        try {
            ResponseResult result = rongCloud.message.msgPrivate.send(message);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云发送单聊自定义消息失败:", e);
        }
        return false;
    }

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param text
     * @param extra
     * @return
     */
    @Override
    public boolean sendToPeerTextMsg(Long userId, Long targetId, String text, String extra) {
        TxtMessage txtMessage = new TxtMessage(text, extra);
        PrivateMessage message = new PrivateMessage()
                .setSenderId(String.valueOf(userId))
                .setTargetId(new String[]{String.valueOf(targetId)})
                .setObjectName(txtMessage.getType())
                .setContent(txtMessage);
        try {
            ResponseResult result = rongCloud.message.msgPrivate.send(message);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云发送单聊文本消息失败:", e);
        }
        return false;
    }

    /**
     * 发送房间消息
     *
     * @param userId
     * @param roomId
     * @param content
     * @return
     */
    @Override
    public boolean sendToChatRoom(Long userId, String roomId, String content) {
        if ("BCR2020".equals(roomId)) {
            roomId = "0";
        }

        CmdMsgMessage cmdMsg = new CmdMsgMessage("ti1mi", content);
        ChatroomMessage message = new ChatroomMessage()
                .setSenderId("0")
                .setTargetId(new String[]{roomId})
                .setObjectName(cmdMsg.getType())
                .setContent(cmdMsg);
        try {
            ResponseResult result = rongCloud.message.chatroom.send(message);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云发送聊天室消息失败:", e);
        }
        return false;
    }

    /**
     * 发送频道消息
     *
     * @param userId
     * @param roomId
     * @param content
     * @param lowPriority
     * @return
     */
    @Override
    public boolean sendToChannel(Long userId, Long roomId, String content, boolean lowPriority) {
        MsgPacket msgPacket = new MsgPacket();
        msgPacket.setMsg(JSON.parseObject(content));
        msgPacket.setGroupId(String.valueOf(roomId));
        msgPacket.setSenderId(userId);
        msgPacket.setTargetId(null);
        msgPacket.setTargetType(ImTargetType.GROUP);
        // 组装用户信息
        if (userId > 0) {
            log.info("组装用户信息-开始");
            msgPacket.setSenderProfile(rtcUserService.getUserProfile(userId, roomId));
            log.info("组装用户信息-结束");
        }

        BaseMessage cmdMsg;
        if (lowPriority) {
            cmdMsg = new RoomJoinMsg("ti1mi", toJSON(msgPacket));
        } else {
            cmdMsg = new CmdMsgMessage("ti1mi", toJSON(msgPacket));
        }

        ChatroomMessage message = new ChatroomMessage()
                .setSenderId("0")
                .setTargetId(new String[]{String.valueOf(roomId)})
                .setObjectName(cmdMsg.getType())
                .setContent(cmdMsg);
        try {
            if (isDev()) {
                log.info("发送消息-开始\n{}", JSON.toJSONString(msgPacket, true));
            }
            ResponseResult result = rongCloud.message.chatroom.send(message);

            if (result.code == 200) {
                log.info("发送消息-成功");
                return true;
            }

            log.info("发送消息-抛出异常错误");
            throw new RuntimeException(toJSON(result));

        } catch (Exception e) {
            log.error("融云发送聊天室消息失败:", e);
        }
        return false;
    }

    /**
     * 聊天室禁言
     *
     * @param roomId
     * @param userId
     * @param minutes
     * @return
     */
    @Override
    public boolean forbiddenChat(Long roomId, Long userId, Integer minutes) {
        ChatroomModel model = new ChatroomModel()
                .setId(String.valueOf(roomId))
                .setMembers(new ChatroomMember[]{new ChatroomMember().setId(String.valueOf(userId))})
                .setMinute(minutes);
        try {
            ResponseResult result = rongCloud.chatroom.gag.add(model);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云禁言失败:", e);
        }
        return false;
    }

    /**
     * 解除禁言
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean enableChat(Long roomId, Long userId) {
        ChatroomModel model = new ChatroomModel()
                .setId(String.valueOf(roomId))
                .setMembers(new ChatroomMember[]{new ChatroomMember().setId(String.valueOf(userId))});
        try {
            ResponseResult result = rongCloud.chatroom.gag.remove(model);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云解除禁言失败:", e);
        }
        return false;
    }

    /**
     * 封禁用户
     *
     * @param roomId
     * @param userId
     * @param minutes
     * @return
     */
    @Override
    public boolean blockRoomUser(Long roomId, Long userId, Integer minutes) {
        ChatroomModel model = new ChatroomModel()
                .setId(String.valueOf(roomId))
                .setMembers(new ChatroomMember[]{new ChatroomMember().setId(String.valueOf(userId))})
                .setMinute(minutes);
        try {
            ResponseResult result = rongCloud.chatroom.block.add(model);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云聊天室封禁用户失败:", e);
        }
        return false;
    }

    /**
     * 解封用户
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean unblockRoomUser(Long roomId, Long userId) {
        ChatroomModel model = new ChatroomModel()
                .setId(String.valueOf(roomId))
                .setMembers(new ChatroomMember[]{new ChatroomMember().setId(String.valueOf(userId))});
        try {
            ResponseResult result = rongCloud.chatroom.block.remove(model);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云聊天室解封用户失败:", e);
        }
        return false;
    }

    /**
     * 设置聊天室属性
     *
     * @param roomId
     * @param key
     * @param value
     * @return
     */
    @Override
    public boolean setRoomProperty(Long roomId, String key, String value) {
        ChatroomEntryModel model = new ChatroomEntryModel()
                .setChatroomId(String.valueOf(roomId))
                .setUserId("0")
                .setKey(key)
                .setValue(value);
        try {
            ResponseResult result = rongCloud.chatroom.entry.set(model);
            if (result.code == 200) {
                return true;
            }
            if (result.code == 23410) {
                // 聊天室不存在
                return false;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云聊天室属性设置失败:", e);
        }
        return false;
    }

    /**
     * 设置聊天室属性
     *
     * @param roomId
     * @param propertyMap
     */
    @Override
    public void setRoomProperties(Long roomId, Map<String, String> propertyMap) {
        for (Map.Entry<String, String> entry : propertyMap.entrySet()) {
            String value = entry.getValue();
            // 空值保护
            if (value == null) {
                continue;
            }
            setRoomProperty(roomId, entry.getKey(), value);
        }
    }

    /**
     * 拉黑用户
     *
     * @param userId
     * @param targetId
     * @return
     */
    @Override
    public boolean addUserBlack(Long userId, Long targetId) {
        UserModel userModel = new UserModel()
                .setId(String.valueOf(userId))
                .setBlacklist(new UserModel[]{new UserModel().setId(String.valueOf(targetId))});
        try {
            Result result = rongCloud.user.blackList.add(userModel);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云用户拉黑失败:", e);
        }
        return false;
    }

    /**
     * 解除拉黑
     *
     * @param userId
     * @param targetId
     * @return
     */
    @Override
    public boolean removeUserBlack(Long userId, Long targetId) {
        UserModel userModel = new UserModel()
                .setId(String.valueOf(userId))
                .setBlacklist(new UserModel[]{new UserModel().setId(String.valueOf(targetId))});
        try {
            Result result = rongCloud.user.blackList.remove(userModel);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云用户解除拉黑失败:", e);
        }
        return false;
    }

    /**
     * 获取拉黑名单
     *
     * @param userId
     * @return
     */
    @Override
    public List<String> getUserBlack(Long userId) {
        UserModel userModel = new UserModel()
                .setId(String.valueOf(userId));
        try {
            BlackListResult result = rongCloud.user.blackList.getList(userModel);
            if (result.code == 200) {
                return Arrays.stream(result.getUsers()).map(UserModel::getId).collect(Collectors.toList());
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云用户拉取黑名单失败:", e);
        }
        return Collections.emptyList();
    }

    /**
     * 单聊禁言
     *
     * @param userId
     * @return
     */
    @Override
    public boolean forbiddenWords(Long userId) {
        return setForbiddenWords(userId, true);
    }

    /**
     * 解除单聊禁言
     *
     * @param userId
     * @return
     */
    @Override
    public boolean cancelForbiddenWords(Long userId) {
        return setForbiddenWords(userId, false);
    }

    /**
     * 获取历史消息
     *
     * @param date
     * @return
     */
    @Override
    public String getHistory(String date) {
        try {
            HistoryMessageResult result = (HistoryMessageResult) rongCloud.message.history.get(date);
            if (result.code == 200) {
                return result.getUrl();
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云获取历史消息失败", e);
        }
        return null;
    }

    /**
     * 获取用户信息
     *
     * @param userId
     * @return
     */
    @Override
    public UserSimpleInfo getUserInfo(Long userId) {
        UserModel userModel = new UserModel()
                .setId(String.valueOf(userId));
        try {
            UserResult userResult = rongCloud.user.get(userModel);
            if (userResult.code == 200) {
                UserSimpleInfo info = new UserSimpleInfo();
                info.setUserId(userId);
                info.setUserName(userResult.getUserName());
                info.setUserAvatar(userResult.getUserPortrait());
                return info;
            }
            throw new RuntimeException(toJSON(userResult));
        } catch (Exception e) {
            log.error("融云获取用户信息失败", e);
        }
        return null;
    }

    private boolean setForbiddenWords(Long userId, boolean forbidden) {
        try {
            StringBuilder sb = new StringBuilder();
            sb.append("userId=").append(URLEncoder.encode(userId.toString(), "UTF-8"));
            sb.append("&state=");
            sb.append(forbidden ? "1" : "0");
            sb.append("&type=PERSON");
            HttpURLConnection conn = HttpUtil.CreatePostHttpConnection(this.rongCloud.getConfig(), this.appKey, this.appSecret, "/user/chat/fb/set.json", "application/x-www-form-urlencoded");
            HttpUtil.setBodyParameter(sb.toString(), conn, this.rongCloud.getConfig());
            ResponseResult result = (ResponseResult) GsonUtil.fromJson(CommonUtil.getResponseByCode("user/blacklist", "add", HttpUtil.returnResult(conn, this.rongCloud.getConfig())), ResponseResult.class);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(toJSON(result));
        } catch (Exception e) {
            log.error("融云禁言用户失败:", e);
        }
        return false;
    }

    /**
     * 对比签名
     *
     * @param timestamp
     * @param nonce
     * @param sign
     * @return
     */
    @Override
    public boolean checkNotifySign(String timestamp, String nonce, String sign) {
        String toSign = String.format("%s%s%s", appSecret, nonce, timestamp);
        String mSign = SecureUtil.sha1(toSign);
        return mSign.equalsIgnoreCase(sign);
    }

    /**
     * 获得房间内在线用户id（最多返回 500 个成员信息）
     * 获取聊天室成员 https://数据中心域名/chatroom/user/query.json
     * https://doc.rongcloud.cn/imserver/server/v1/chatroom/info#user_query
     *
     * @param roomId
     * @return 房间内在线用户id
     * @throws Exception
     */
    @Override
    public List<Long> onlineUserIdsInRoom(Long roomId) throws Exception {
        if (roomId == null) {
            return Collections.emptyList();
        }

        ChatroomModel chatroomModel = new ChatroomModel()
                .setId(String.valueOf(roomId))
                .setCount(500)
                .setOrder(2);
        return rongCloud.chatroom.get(chatroomModel).getMembers().stream()
                .map(user -> NumberUtils.toLong(user.id))
                .distinct()
                .collect(Collectors.toList());
    }

    private static String toJSON(Object obj) {
        return JSON.toJSONString(obj, SerializerFeature.WriteNonStringKeyAsString, SerializerFeature.DisableCircularReferenceDetect);
    }

    //---以下为老版腾讯im的封装接口---
    /**
     * 创建 IM 用户
     *
     * @param imUserCreateDTO
     * @return
     */
    @Override
    public String createUser(ImUserCreateDTO imUserCreateDTO) {
        return null;
    }

    /**
     * 刷新 Token
     *
     * @param userId
     * @param expireTime
     * @return
     */
    @Override
    public String refreshToken(Long userId, Long expireTime) {
        return null;
    }

    /**
     * 创建直播间聊天室
     *
     * @param userId
     * @param roomId
     * @return
     */
    @Override
    public String addLiveChatRoom(Long userId, String roomId) {
        return null;
    }

    /**
     * 同步用户资料
     *
     * @param imProfile
     * @return
     */
    @Override
    public boolean syncUserProfile(ImUserProfile imProfile) {
        return false;
    }

    /**
     * 发送自定义消息
     *
     * @param senderId
     * @param targetId
     * @param imMsg
     * @param imPushInfo
     * @return
     */
    @Override
    public boolean sendCustomImMsg(Long senderId, Long targetId, BaseImMsg imMsg, ImPushInfo imPushInfo) {
        return false;
    }

    /**
     * 发送直播间消息
     *
     * @param anchorUserId
     * @param fromUserId
     * @param imMsg
     * @param msgPriority
     * @return
     */
    @Override
    public boolean sendLiveRoomMsg(Long anchorUserId, Long fromUserId, BaseImMsg imMsg, String msgPriority) {
        return false;
    }

    /**
     * 发送广播消息
     *
     * @param channelId
     * @param fromUserId
     * @param imMsg
     * @param msgPriority
     * @return
     */
    @Override
    public boolean sendBroadcast(Long channelId, Long fromUserId, BaseImMsg imMsg, String msgPriority) {
        return false;
    }

    /**
     * 同步群成员资料
     *
     * @param anchorUserId
     * @param imGroupUserProfile
     * @return
     */
    @Override
    public boolean syncLiveChatRoomUserProfile(Long anchorUserId, LiveChatRoomUserProfile imGroupUserProfile) {
        return false;
    }

    /**
     * 获取app信息
     *
     * @return
     */
    @Override
    public TencentImBizGetAppInfoResponse getAppInfo() {
        return null;
    }

    /**
     * 发送文本消息
     *
     * @param senderId
     * @param targetId
     * @param text
     * @return
     */
    @Override
    public boolean sendTextMsg(Long senderId, Long targetId, String text) {
        return false;
    }
}
