package com.starbuds.server.service.message.tencent.callback;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.db.PublicChatroomMsgHisDO;
import com.starbuds.server.common.pojo.mq.QueueLiveUserExitMsg;
import com.starbuds.server.common.pojo.mq.QueueLiveUserJoinMsg;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.message.dao.PublicChatRoomMsgHisDAO;
import com.starbuds.server.service.message.rocket.RocketProducer;
import com.starbuds.server.service.message.tencent.callback.constant.CallbackCommand;
import com.starbuds.server.service.message.tencent.callback.constant.StateChangeInfoAction;
import com.starbuds.server.service.message.tencent.callback.entity.CallbackEntity;
import com.starbuds.server.service.message.tencent.callback.entity.group.CallbackAfterSendMsgEntity;
import com.starbuds.server.service.message.tencent.callback.entity.group.MemberExitEntity;
import com.starbuds.server.service.message.tencent.callback.entity.group.NewMemberJoinEntity;
import com.starbuds.server.service.message.tencent.callback.entity.state.StateChangeEntity;
import com.starbuds.server.service.message.tencent.constant.TencentGroupType;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 回调处理
 *
 * @author Lay
 * @date 2019/12/30
 */
@Component
@Slf4j
public class TencentCallbackHandler {

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    PublicChatRoomMsgHisDAO publicChatRoomMsgHisDAO;

    @Resource
    RocketProducer rocketProducer;

    /**
     * 回调处理
     *
     * @param callbackCommand
     * @param clientIp
     * @param optPlatform
     * @param content
     * @return
     */
    public boolean handleCallback(String callbackCommand, String clientIp, String optPlatform, String content) {
        log.info("处理腾讯 IM 回调: {}", callbackCommand);
        switch (callbackCommand) {
            case CallbackCommand.STATE_STATE_CHANGE:
                StateChangeEntity stateChangeEntity = parseCommandEntity(callbackCommand, clientIp, optPlatform, content, StateChangeEntity.class);
                return handleStateChangeCallback(stateChangeEntity, clientIp, optPlatform);
            case CallbackCommand.GROUP_CALLBACK_AFTER_SEND_MSG:
                CallbackAfterSendMsgEntity callbackAfterSendMsgEntity = parseCommandEntity(callbackCommand, clientIp, optPlatform, content, CallbackAfterSendMsgEntity.class);
                return handleGroupSendMsgCallback(callbackAfterSendMsgEntity);
            default:
                return false;
        }
    }

    private static <T extends CallbackEntity> T parseCommandEntity(String callbackCommand, String clientIp, String optPlatform, String content, Class<T> tClass) {
        T t = JSON.parseObject(content, tClass);
        t.setCallbackCommand(callbackCommand);
        t.setClientIp(clientIp);
        t.setOptPlatform(optPlatform);
        return t;
    }

    /**
     * 处理状态变化
     *
     * @param stateChangeEntity
     * @return
     */
    private boolean handleStateChangeCallback(StateChangeEntity stateChangeEntity, String clientIp, String platform) {
        // 修改用户在线状态
        Long userId = Convert.toLong(stateChangeEntity.getInfo().getToAccount());
        Integer isOnline = StateChangeInfoAction.LOGIN.equals(stateChangeEntity.getInfo().getAction()) ? BoolIntVal.True : BoolIntVal.False;

        return userService.setIsOnline(userId, isOnline, clientIp, platform);
    }

    /**
     * 存储消息
     *
     * @param entity
     * @return
     */
    private boolean handleGroupSendMsgCallback(CallbackAfterSendMsgEntity entity) {
        Long fromUserId = Convert.toLong(entity.getFromAccount());
        if (fromUserId == null || fromUserId == 0) {
            return true;
        }
        PublicChatroomMsgHisDO msgHisDO = new PublicChatroomMsgHisDO();
        msgHisDO.setMsgId(entity.getMsgSeq());
        msgHisDO.setGroupId(entity.getGroupId());
        msgHisDO.setMsgBody(entity.getMsgBody());
        msgHisDO.setFromUserId(fromUserId);
        msgHisDO.setMsgTime(entity.getMsgTime());
        publicChatRoomMsgHisDAO.insert(msgHisDO);
        return true;
    }

    /**
     * 处理用户加入群组
     *
     * @param newMemberJoinEntity
     * @return
     */
    private boolean handleAfterGroupMemberJoin(NewMemberJoinEntity newMemberJoinEntity) {
        if (TencentGroupType.AVCHATROOM.equals(newMemberJoinEntity.getType())) {
            // 新成员进入直播间
            Long anchorUserId = Convert.toLong(newMemberJoinEntity.getGroupId().replace("LCR", ""));

            for (NewMemberJoinEntity.Member member : newMemberJoinEntity.getNewMemberList()) {
                Long joinUserId = Convert.toLong(member.getMemberAccount());
                QueueLiveUserJoinMsg liveUserJoinMsg = new QueueLiveUserJoinMsg();
                liveUserJoinMsg.setAnchorUserId(anchorUserId);
                liveUserJoinMsg.setJoinUserId(joinUserId);
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LIVE_USER_JOIN, liveUserJoinMsg.serialize());
            }
            return true;
        }

        return false;
    }

    /**
     * 处理用户离开群组
     *
     * @param memberExitEntity
     * @return
     */
    private boolean handleAfterGroupMemberExist(MemberExitEntity memberExitEntity) {
        if (TencentGroupType.AVCHATROOM.equals(memberExitEntity.getType())) {
            // 离开直播间
            Long anchorUserId = Convert.toLong(memberExitEntity.getGroupId().replace("LCR", ""));
            for (MemberExitEntity.Member member : memberExitEntity.getExitMemberList()) {
                Long exitUserId = Convert.toLong(member.getMemberAccount());
                QueueLiveUserExitMsg liveUserExitMsg = new QueueLiveUserExitMsg();
                liveUserExitMsg.setAnchorUserId(anchorUserId);
                liveUserExitMsg.setExitUserId(exitUserId);
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LIVE_USER_EXIT, liveUserExitMsg.serialize());
            }
            return true;
        }
        return false;
    }
}
